%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main function for figure 10. (a)
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clc ; 
clear all ;
format long ;

disp(datestr(now))


Ls_set = 0 : 5: 30 ;
Lr_set = 0 : 5: 20 ;
Ls_set(1) = 1 ;
Lr_set(1) = 1 ;

s = 1  % noise std deviation
N =2^7  % number of Riemann partitions or OFDM carriers

Ps = 1 ; % source power level
Pr = 1 ; % relay power level

epsilon = 1e-5 ;
max_iter =  500  ;  % subgradient iteration
lambda =1 ; % diminishing step size ; lambda/sqrt(iter)
channel_index = 100 ;  % channel averaging
L = 5 ; % delay spread


%% channel generation
H = randn(L,3,length(channel_index)) ;

H(:,1,:) = H(:,1,:) * sqrt(0.1) ; % h_sd ; varying
H(:,2,:) = H(:,2,:) * 1; % h_sr ; fixed at all frames
H(:,3,:) = H(:,3,:) * sqrt(10) ; % h_rd ; varying



tic;

rate_table = zeros( length(Ls_set) , length(Lr_set) ) ;

for id_Ls = 1: length(Ls_set)
    for id_Lr=1 : length(Lr_set)

        Ls = Ls_set(id_Ls) ; 
        Lr = Lr_set(id_Lr) ; 
        
        for id_monte = 1 : length(channel_index)
            h_sd = H(:,1,id_monte) ;  
            h_sr = H(:,2,id_monte) ;  
            h_rd = H(:,3,id_monte) ;

            tap_set = [Ls ; Lr ] ;
            u_initial = zeros(Ls+Lr,1) ;
            u_initial(1) = sqrt(Ps/Ls); 
            
            parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial  ] ;
            [ rate , u_opt ] = func_algorithm2(parameters ) ; % algorithm 2
            ind_rate = find(rate, 1, 'last') ; % last index
            
            rate_table(id_Ls,id_Lr) =rate_table(id_Ls,id_Lr) + rate(ind_rate)/ length(channel_index) ;

            toc ;
            
        end
    end
end


figure(101) ; clf ;
    mesh(Ls_set, Lr_set, rate_table')
    title('Number of filter taps and the achievable rate, P_s=P_r=1')
    xlabel('L_s (Number of source filter taps)')
    ylabel('L_r (Number of source filter taps)')
    zlabel('Achievable rate (bit/transmission)')
    
    
    
    
