


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main function for figure 10. (b)
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear all ;

disp(datestr(now))

%% parameters
velocity = 3e3/3600 ; % destination speed in km/hour
carrier_freq = 1e9 ; % center freq
freq_Doppler = velocity / 3e8 * carrier_freq ; % Doppler freq
Frame_length = 66.7e-6 ; % in sec
num_sym = 100 ; % number of symbols in one frame
rho = besselj(0,2*pi* freq_Doppler * num_sym*Frame_length) ; % correlated block fading channel, AR cofficients

L = 5 ; % delay spread

s = 1 ; % noise std deviation
N =2^7 ; % number of Riemann partitions or OFDM carriers

Ps = 1 ; % source power level
Pr = 1 ; % relay power level

epsilon =1e-6; % stopping relative error
max_iter =  1000 ;  % maximum number of subgradient iteration

lambda = 1 ; % diminishing step size ; lambda/sqrt(iter)
options = optimset('Algorithm','interior-point', 'Display','iter-detailed','MaxFunEvals',1e4) ;

Ls = 10 ; % number of taps for source filter
Lr = 10 ; % number of taps for relay filter
tap_set = [Ls ; Lr ] ;
max_frame = 50 ; % number of frame to be observed

%% arrays
convergence_PSM = zeros(max_frame,1) ;
convergence_interior = zeros(max_frame,1) ;

rate_table_APSM_1 = zeros(max_iter, max_frame ) ;
rate_table_APSM_5 = zeros(max_iter, max_frame ) ;
rate_table_APSM_many= zeros(max_iter, max_frame ) ;
rate_table_PSM = zeros(max_iter, max_frame ) ;
rate_table =zeros(5 , max_frame ) ; % [APSM ; PSM ; fmincon ]

% initial conditions
u_initial_PSM = zeros(Ls+Lr,1) ;
u_initial_PSM(1) = sqrt(Ps/Ls);   

u_initial_APSM=u_initial_PSM ;
u_initial_APSM_1 = u_initial_APSM ;
u_initial_APSM_5 = u_initial_APSM ;
u_initial_APSM_many = u_initial_APSM ;

u_initial_interior = u_initial_PSM ;
u0 = u_initial_PSM ;

%% channel generation

% H = zeros(L,3,max_frame) ;
% H(:,:,1) = randn(L,3) ;
% for id = 2 : max_frame % h_sr is fixed; Only the destination is moving
%     H(:,1,id) = rho*H(:,1,id-1) + sqrt(1-rho^2)*randn(L,1,1) ; % h_sd
%     H(:,3,id) = rho*H(:,3,id-1) + sqrt(1-rho^2)*randn(L,1,1) ; % h_rd
% end
% 
% H(:,1,:) = H(:,1,:) * sqrt(0.1) ; % h_sd
% H(:,2,:) = H(:,2,:) * 1 ; % h_sr
% H(:,3,:) = H(:,3,:) * sqrt(10) ; % h_rd

load H_9966

%% programs
tic ; 

for id_frame = 1 : max_frame 

        h_sd = H(:,1,id_frame) ;
        h_sr = H(:,2,id_frame) ;
        h_rd = H(:,3,id_frame) ;
    
        Ls = tap_set(1) ; Lr = tap_set(2) ;

        fprintf('\nPSM at every frame\n\n')
        parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u0 ] ;
        [ rate_PSM, dumm ]  = func_algorithm2(parameters) ;% 1 plot on
        rate_table_PSM(: , id_frame) = rate_PSM ;

        fprintf('\nAPSM\n\n')
        if id_frame ==1 % precise calculation
            max_ind_PSM = find(rate_PSM, 1, 'last') ;
            rate_table_APSM_1(: , id_frame)  = rate_PSM(max_ind_PSM) ; 
            rate_APSM_1= rate_PSM(max_ind_PSM) ;
            
            rate_table_APSM_5(: , id_frame)  = rate_PSM(max_ind_PSM) ;            
            rate_APSM_5 = rate_PSM(max_ind_PSM) ;
            
            rate_table_APSM_many(: , id_frame)  = rate_PSM(max_ind_PSM) ;            
            rate_APSM_many = rate_PSM(max_ind_PSM) ;
            
        else
            
            % update once
            parameters = [ tap_set ; s ; N ; epsilon ; 1 ; lambda*2 ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial_APSM_1  ] ;
            [ rate_APSM_1, u_opt ]  = func_algorithm2(parameters ) ;
            rate_table_APSM_1(: , id_frame) = [ rate_APSM_1 ; zeros(max_iter - length(rate_APSM_1) , 1 ) ]  ;
            u_initial_APSM_1 = u_opt ;     
            
            % update five times 
            parameters = [ tap_set ; s ; N ; epsilon ; 5 ; lambda*1.5 ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial_APSM_5  ] ;
            [ rate_APSM_5, u_opt ]  = func_algorithm2(parameters ) ;
            rate_table_APSM_5(: , id_frame) = [ rate_APSM_5 ; zeros(max_iter - length(rate_APSM_5) , 1 ) ]  ;
            u_initial_APSM_5 = u_opt ;     
            
            % update until converges
            parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial_APSM_many  ] ;
            [ rate_APSM_many, u_opt ]  = func_algorithm2(parameters ) ;
            rate_table_APSM_many(: , id_frame) = [ rate_APSM_many ; zeros(max_iter - length(rate_APSM_many) , 1 ) ]  ;
            u_initial_APSM_many = u_opt ;                 
            
            
        end
        
        
        fprintf('\nInterior point at every frame\n\n')
        parameters = [ Ps ; Pr ; N ; h_sd ; h_sr ; h_rd ; s ; Ls ; Lr ; L ] ;
        AA=[] ;        bb=[];        Aeq=[];        beq=[]; lb=[] ; ub=[] ;        
        [u_opt2, rate_interior, exitflag, output] = fmincon(@(u) obj(u, parameters), u_initial_interior,AA, bb, Aeq, beq, lb, ub, @(u) nonlcon(u, parameters) , options) ;

        
        max_ind_APSM_5 = find(rate_APSM_5, 1, 'last') ;
        max_ind_PSM = find(rate_PSM, 1, 'last') ;
        max_ind_APSM_many= find(rate_table_APSM_many, 1, 'last') ;
        
        toc ;
                
        convergence_PSM(id_frame) = max_ind_PSM ;
        convergence_interior(id_frame) = output.iterations ;
        
        rate_table(: , id_frame) = [rate_APSM_1 ; rate_APSM_5(max_ind_APSM_5) ; rate_table_APSM_many(max_ind_APSM_many)  ; rate_PSM(max_ind_PSM) ; -rate_interior] ;

end



%% reports and plots


figure ;
hold on ;
    plot(1:max_frame , rate_table(1,:),'r-s' ) ;
    plot(1:max_frame , rate_table(2,:),'b-^' ) ;
    plot(1:max_frame , rate_table(4,:),'k-+' ) ;
    plot(1:max_frame , rate_table(5,:),'k-o' ) ;
hold off ;

xlabel('Frame')
ylabel('Rate (bit/transmission)')
legend('APSM : only one update','APSM : five updates','PSM','Interior-point method',4)
box on ;
