%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main function for figure 8. (a) (b)
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clc ; 
clear all ;
format long ;

disp(datestr(now))


Ls=30 ; % number of taps for source filter
Lr=20 ; % number of taps for relay filter

s = 1 ; % noise std deviation
N =2^9 ; % number of Riemann partitions or OFDM carriers

Ps_set = 1 ; % power level

epsilon = 1e-5 ;
max_iter =  500  ;  % subgradient iteration
lambda =1 ; % diminishing step size ; lambda/sqrt(iter)
L = 5 ; % delay spread

%% channel generation

h_sd = [ 
  -0.886373886104456 ;
  -1.840197110657360 ;
  -1.628195157394382 ;
  -1.173788621940610 ;
  -0.415410884671374 ] ;

h_sr = [
   1.883290751835338 ;
   0.325434625888199 ;
  -0.095212709715086 ;
   0.031171649072497 ;
  -0.613799565082297 ] ;

h_rd = [
  -0.072770059812388 ;
   1.314783773259100 ;
   0.978279424201783 ; 
   1.722114153403701 ;
  -0.412301793673177 ] ;
  
  
tic;

Ps = 1 ;
Pr = 1 ;

%%% LTI filtering
tap_set = [Ls ; Lr ] ;
u_initial = zeros(Ls+Lr,1) ;
u_initial(1) = sqrt(Ps); 
u_initial(Ls+1:end) = ones(Lr,1) ;
            
parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial  ] ;
[ rate , u_opt_LTI ] = func_algorithm2(parameters ) ; % algorithm 2
toc ;
 
T_LTI= fft(u_opt_LTI(1:Ls),N) ;
H_LTI =fft(u_opt_LTI(1+Ls:end),N) ;




%%% AF
tap_set = [1 ; 1 ] ;
u_initial = zeros(Ls+Lr,1) ;
u_initial(1) = sqrt(Ps); 
u_initial(Ls+1:end) = ones(Lr,1) ;

u_initial_C1 = func_project_onto_C1(Ps, Ls ,u_initial ) ;
[ u_initial_C1_C2, flag] = function_project_onto_C2(u_initial_C1, Pr, s, Ls, Lr, N, h_sr) ; % projection to C2
u_initial_C1_C2 = real(u_initial_C1_C2) ;
            
parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial_C1_C2  ] ;
[ rate , u_opt ] = func_algorithm2(parameters ) ; % algorithm 2
toc ;
 
T_AF= fft(u_opt(1),N) ;
H_AF =fft(u_opt(2),N) ;

%%% plotting fig 8 (a)

% LTI case
H_sd = fft([h_sd ; 0],N) ;
H_sr = fft([h_sr;0],N) ;
H_rd = fft([h_rd;0],N) ;

if (size(H_LTI)~=size(H_sd))
    H_LTI=transpose(H_LTI) ;
end
if (size(T_LTI) ~=size(H_sd))
    T_LTI=transpose(T_LTI) ;
end
CNR_LTI = (abs(H_sd + H_sr .* H_LTI .* H_rd).^2) ./ (( abs(H_rd.* H_LTI).^2+1)*s^2 ) ;                                                            

% AF case
if (size(H_AF)~=size(H_sd))
    H_AF=transpose(H_AF) ;
end
if (size(T_AF) ~=size(H_sd))
    T_AF=transpose(T_AF) ;
end
CNR_AF = (abs(H_sd + H_sr .* H_AF .* H_rd).^2) ./ (( abs(H_rd.* H_AF).^2+1)*s^2 ) ;                                                            


% freq
w = linspace(0,2*pi,N+1) ; 
w = w(1:N) ;

figure; hold on ;                                                            
    plot(w/pi ,1./(abs(CNR_AF)) ,'Color',[149/256 69/256 19/256 ],'LineStyle','-.')
    plot(w/pi ,abs(T_AF).^2  ,'Color',[0 0 0 ],'LineStyle','--')
    plot(w/pi ,1./(abs(CNR_LTI)) , 'Color',[1 0 0 ],'LineStyle','-','Marker','+')
    plot(w/pi ,abs(T_LTI).^2, 'Color',[0 0 1 ],'LineStyle','-','Marker','x')
    plot(w/pi ,abs(T_LTI).^2 +1./(abs(CNR_LTI)) ,'k-')
hold off ;
    
legend('1/CNR_{AF}(e^{j\omega})','|T_{AF}(e^{j\omega})|^2','1/CNR(e^{j\omega})','T(e^{j\omega})|^2','|1/CNR(e^{j\omega})+T(e^{j\omega})|^2',2)
xlabel('\omega_c/\pi') ;
ylabel('Magnitude')
axis([0 1 0 10])
box on ;


%%% plotting fig 8 (b)

figure; 
t = u_opt_LTI(1:Ls) ;
h = u_opt_LTI(Ls+1:end) ;
subplot(2,1,1) ; 
    stem(t) ; xlabel('Tap index'); ylabel('Value') ; title('The impulse response of the source filter t[n]')
subplot(2,1,2) ; 
    stem(h) ; xlabel('Tap index'); ylabel('Value') ; title('The impulse response of the relay filter h[n]')



