%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main function for figure 9. (a) (b) (c) (d)
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc ; 
clear all ;
format long ;

disp(datestr(now))


Ls=30 ; % number of taps for source filter
Lr=20 ; % number of taps for relay filter

s = 1 ; % noise std deviation
N =2^7 ; % number of Riemann partitions or OFDM carriers

Ps_set = logspace(-2 , log10(1), 20) ; % power level

epsilon = 1e-5 ;
max_iter =  500  ;  % subgradient iteration
lambda =1 ; % diminishing step size ; lambda/sqrt(iter)
channel_index = [ 1:1 ] ; % channel averaging
L = 5 ; % delay spread



%%% Figure 9 (a)
scale = [ 1 ; 1 ; 1 ] ;

% %%% Figure 9 (b)
% scale = [ 1 ; sqrt(4) ; 1 ] ;
% 
% %%% Figure 9 (c)
% scale = [ 1 ; 1 ; sqrt(4) ] ;
% 
% %%% Figure 9 (d)
% scale = [ 1/sqrt(10) ; 1 ; sqrt(10) ] ;


%% channel generation

H = randn(L,3,length(channel_index)) ;
   

tic;

rate_causal = zeros( length(Ps_set) ,1 ) ;
rate_strictly = zeros( length(Ps_set) ,1 ) ;
rate_AF = zeros( length(Ps_set) ,1 ) ;

for id_power = 1 : length(Ps_set)
    Ps = Ps_set(id_power) ;
    Pr = Ps_set(id_power) ;
    
    for id_monte = 1 : length(channel_index)

            h_sd = H(:, 1, id_monte) * scale(1) ;  
            h_sr = H(:, 2, id_monte)  * scale(2);  
            h_rd = H(:, 3, id_monte) * scale(3);

            % causal case
            tap_set = [Ls ; Lr ] ;
            u_initial_causal = zeros(Ls+Lr,1) ;
            u_initial_causal(1) = sqrt(Ps/Ls); 
            
            parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial_causal  ] ;
            [ rate , u_opt ] = func_algorithm2(parameters ) ; % algorithm 2
            ind_rate = find(rate, 1, 'last') ; % last index
            rate_causal(id_power) =rate_causal(id_power) + rate(ind_rate)/ length(channel_index) ;
            toc ;
           
            % strictly causal case
            tap_set = [Ls ; Lr ] ;
            u_initial_strictly = zeros(Ls+Lr,1) ;
            u_initial_strictly(2) = sqrt(Ps) ;
            u_initial_strictly(Ls+1:end) = ones(Lr,1) ;
            u_initial_C1 = func_project_onto_C1(Ps, Ls ,u_initial_strictly ) ;
            [ u_initial_C1_C2, flag] = function_project_onto_C2_strictly(u_initial_C1, Pr, s, Ls, Lr, N, h_sr) ; % projection to C2
            u_initial_C1_C2 = real(u_initial_C1_C2) ;
            
            parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial_C1_C2  ] ;
            [ rate , u_opt ] = func_algorithm2_strict(parameters ) ; % algorithm 2
            ind_rate = find(rate, 1, 'last') ; % last index
            rate_strictly(id_power) =rate_strictly(id_power) + rate(ind_rate)/ length(channel_index) ;
            toc ;            

            % AF case
            u_initial = zeros(Ls+Lr,1) ;
            u_initial(1) = sqrt(Ps/Ls); 
            parameters = [ tap_set ; s ; N ; epsilon ; max_iter ; lambda ; L ; h_sr ; h_sd ; h_rd ;  Ps ; Pr ; u_initial  ] ;
            [ rate , u_opt ] = func_algorithm2(parameters ) ; % algorithm 2
            ind_rate = find(rate, 1, 'last') ; % last index
            rate_AF(id_power) =rate_AF(id_power) + rate(ind_rate)/ length(channel_index) ;
            toc ;

    end

end

figure; clf ; hold on ;
    plot( log10(Ps_set) , rate_causal, 'r-o')
    plot( log10(Ps_set), rate_strictly, 'b-+')
    plot( log10(Ps_set), rate_AF, 'k--')
hold off ;
legend('Proposed - causal (30,20)','Proposed - strictly causal (30,20)','AF (1,1)',2) ;
xlabel('Transmit power (P_s=P_r)')
ylabel('Rate (bit/transmission)')
box on ;


