%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Constraint functions
%             Cheulsoon Kim and Youngchul Sung
%                             Jan. 6, 2012
%             
%  Deailed explanation is in the following paper by Cheulsoon Kim and Youngchul Sung, 
% "A Joint Time-Invariant Filtering Approach to the Linear Gaussian
%  Relay Problem," submitted to IEEE Transactions on Signal Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input : 
%         Ps : Source power
%         Pr : Relay power
%         N 	: Number of partition to calculate the integration
%         L : Delay spread
%         h_sr : FIR channel between the source and the relay; (L) by (1)
%         h_sd : FIR channel between the source and the destination; (L) by (1)
%         h_rd : FIR channel between the relay and the destination; (L) by (1)
%         s     : Noise standard deviation
%         Ls  : Number of source filter taps
%         Lr  : Number of relay filter taps
% Output :      
%         c    :  [The source power - Ps ; The relay power - Pr ]
%         ceq    :  []

function [c, ceq] = nonlcon(u, parameters) 


% parameters = [ Ps ; Pr ; N ; h_sd ; h_sr ; h_rd ; s ; Ls ; Lr ; L ] ;

Ps = parameters(1) ;
Pr = parameters(2);
N = parameters(3) ;
L = parameters(end) ;
h_sd = parameters(4:3+L) ;
h_sr = parameters(4+L:3+2*L) ;
h_rd = parameters(4+2*L:3+3*L) ;
s = parameters(end-3) ;
Ls = parameters(end-2) ;
Lr = parameters(end-1) ;


t = u(1:Ls) ;
h = u(Ls+1:end) ;


H_sd = fft([h_sd;0],N) ;
H_sr = fft([h_sr;0],N) ;
H_rd = fft([h_rd;0],N) ;

H = fft([h;0], N) ;
T= fft([t;0],N) ;

if  size(H) ~=size(H_sd)
    H = transpose(H) ;
end



c=zeros(2,1) ;

c(1) = transpose(t)*t - Ps ;

if size(H_sr) ~= size(T)
    T = transpose(T) ;
end

tmp1 = (abs(H).^2) ;
tmp2 = (abs(H_sr)).^2 ;
tmp3 = (abs(T)).^2 ;
tmp4 = (tmp2 .* tmp3+s^2) ;

integrand = tmp1.*tmp4 ;
c(2) = sum(integrand)/N - Pr ;

ceq = [] ;




end
