%%%%%%
%%% Projection onto C2
%%% 2011-07-11 Cheulsoon
%%% 2011-07-15 Cheulsoon version 2 (quad integration)
%%%%%%

%%% input ; Q ; square positive semidefinite symmetric
%%%          ; u ; vector in to be projected

%%% output ; h0 ; metric projection


function [uu , flag] = project_onto_C2_v2(u, Pr, s, Ls, Lr, N, h_sr)


t = u(1:Ls) ; % source filter
h = u(Ls+1 : end) ; % relay filter


[w_sampled,weight] = lgwt(N, 0, 2*pi ) ; % freq bins to use quadrature
% w_sampled=sort(w_sampled) ;
% weight=sort(weight) ;

dft_mat_quad = exp(1i).^ [ transpose([0:N-1]) * transpose(w_sampled) ] ; % dft matrix at sampling points on the unit circle

H_sr = ctranspose(dft_mat_quad) * [h_sr ; zeros(N-length(h_sr),1) ] ; % freq representation of h_sr
T = ctranspose(dft_mat_quad) * [t ; zeros(N-length(t),1) ] ; % freq representation of t
H =  ctranspose(dft_mat_quad) * [h ; zeros(N-length(h),1) ] ; % freq representation of h

aa = (abs(H_sr).^2) .* (abs(T).^2) + s^2 ; % alpha array

integrand = (abs(H).^2).*((abs(H_sr).^2).*(abs(T).^2) +s^2) ; % integrand of constraint 2

if  sum(integrand.*weight)/2/pi < Pr  
    uu = u ; % if u is feasible, then stay still.
    flag_inner = 0 ;
else % if u is not feasible, then project.

    integrand_quad = zeros( Lr) ; % array quad for integrand2
    
    for id_bin = 1 : N
        
        w_val = w_sampled(id_bin) ; % sampled point
        w_h=exp(1i*w_val*transpose([0:Lr-1]) ) ; % quasi-dft vector        
        integrand_quad = integrand_quad + w_h * ctranspose(w_h) * aa(id_bin) * weight(id_bin) ; % weight sum for quad integration
        
    end
   
    Q_quad = integrand_quad/2/pi ;  % ellipse equation
 
    [h0, flag_inner] = project_onto_ellipse(Q_quad/Pr ,h) ; % the actual projection

    uu = u ;
    uu(Ls+1 : end) = h0 ; % update h0

    
end

flag = flag_inner ;
    

end



